<?php

/**
 * @file
 * Invoice administration menu items.
 */

/**
 * Creates a new invoice and redirect to its edit screen.
 * For Creates a form for creating a invoice.
 * @, see uc_payafter_invoice_create_form_validate()
 * @, see uc_payafter_invoice_create_form_submit()
 * @ingroup forms
 */
function uc_payafter_invoice_create_form($form, &$form_state) {
  $form['customer_type'] = array(
    '#type' => 'radios',
    '#options' => array(
      'search' => t('Search for an existing customer.'),
      'create' => t('Create a new customer account.'),
    ),
    '#required' => TRUE,
    '#ajax' => array(
      'callback' => 'uc_payafter_create_form_customer',
      'wrapper'  => 'custom-payafter-customer',
      'progress' => array('type' => 'throbber'),
    ),
  );

  $form['customer'] = array(
    '#prefix' => '<div id="custom-payafter-customer">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );

  // Create form elements needed for customer search.
  // Shown only when the 'Search for an existing customer.' radio is selected.
  if (!isset($form_state['values']['customer_type']) ||
      $form_state['values']['customer_type'] == 'search') {

    // Container for customer search fields.
    $form['customer'] += array(
      '#type' => 'fieldset',
      '#title' => t('Customer search'),
      '#description' => t('Enter full or partial information in one or more of the following fields, then press the "Search" button. Search results will match all the provided information.'),
    );
    // Customer e-mail address.
    $form['customer']['email'] = array(
      '#type' => 'textfield',
      '#title' => t('E-mail'),
      '#size' => 24,
      '#maxlength' => 96,
    );
    // Customer username.
    $form['customer']['username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#size' => 24,
      '#maxlength' => 96,
    );
    $form['customer']['search'] = array(
      '#type' => 'button',
      '#value' => t('Search'),
      '#ajax' => array(
        'callback' => 'uc_payafter_create_form_customer_search',
        'wrapper' => 'custom-payafter-customer-results',
        'progress' => array('type' => 'throbber'),
      ),
    );
    $form['customer']['uid'] = array(
      '#prefix' => '<div id="custom-payafter-customer-results">',
      '#suffix' => '</div>',
    );

    // Search for existing customer by e-mail address.
    if (isset($form_state['values']['customer']['email'])) {
      $query = db_select('users', 'u')->distinct();
      $query->fields('u', array('uid', 'name', 'mail'))
        ->condition('u.uid', 0, '>')
        ->condition(db_or()
          ->condition('u.mail', db_like(trim($form_state['values']['customer']['email'])) . '%', 'LIKE')
        )
        ->condition('u.name', db_like(trim($form_state['values']['customer']['username'])) . '%', 'LIKE')
        ->orderBy('u.created', 'DESC')
        ->range(0, $limit = 11);

      $result = $query->execute();

      $options = array();
      foreach ($result as $user) {
        $name = $user->name;
        // Options formatted as "First Last <email@example.com> (username)".
        $options[$user->uid] = $name . '&lt;' . $user->mail . '&gt;' . ' (' . $user->name . ')';
      }

      if (count($options) == $limit) {
        array_pop($options);
      }

      if (!empty($options)) {
        // Display search results.
        $form['customer']['uid'] += array(
          '#type' => 'radios',
          '#title' => t('Select customer'),
          '#description' => t('More than !limit results found. Refine your search to find other customers.', array('!limit' => $limit - 1)),
          '#options' => $options,
          '#default_value' => key($options),
        );
      }
      else {
        // No search results found.
        $form['customer']['uid'] += array(
          '#markup' => '<p>' . t('Search returned no results.') . '</p>',
        );
      }
    }
  }
  // Create form elements needed for new customer creation.
  // Shown only when the 'Create a new customer account.' radio is selected.
  elseif ($form_state['values']['customer_type'] == 'create') {
    // Container for new customer information.
    $form['customer'] += array(
      '#type'  => 'fieldset',
      '#title' => t('New customer details'),
    );
    // Customer e-mail address.
    $form['customer']['email'] = array(
      '#type' => 'textfield',
      '#title' => t('Customer e-mail address'),
      '#size' => 24,
      '#maxlength' => 96,
    );
    // Option to notify customer.
    $form['customer']['sendmail'] = array(
      '#type'  => 'checkbox',
      '#title' => t('E-mail account details to customer.'),
    );
  }

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create Invoice'),
    '#validate' => array('uc_payafter_invoice_create_form_validate'),
    '#submit' => array('uc_payafter_invoice_create_form_submit'),
  );

  return $form;
}


/**
 * Ajax callback: updates the customer search results.
 */
function uc_payafter_create_form_customer_search($form, &$form_state) {
  return $form['customer']['uid'];
}

/**
 * Ajax callback: updates the customer selection fields.
 */
function uc_payafter_create_form_customer($form, &$form_state) {
  return $form['customer'];
}

/**
 * Form validation handler for customer search.
 *
 * @, see uc_payafter_create_form()
 * @ingroup forms
 */
function uc_payafter_invoice_create_form_validate($form, &$form_state) {
  if ($form_state['values']['customer_type'] == '') {
    form_set_error('customer][uid', t('Please select the customer type.'));
  }
  switch ($form_state['values']['customer_type']) {
    case 'search':
      if (empty($form_state['values']['customer']['uid'])) {
        form_set_error('customer][uid', t('Please select a customer.'));
      }
      break;

    case 'create':
      $email = trim($form_state['values']['customer']['email']);
      if (!valid_email_address($email)) {
        form_set_error('customer][mail', t('Invalid e-mail address.'));
      }

      $uid = db_query('SELECT uid FROM {users} WHERE mail LIKE :mail', array(':mail' => $email))->fetchField();
      if ($uid) {
        form_set_error('customer][mail', t('An account already exists for that e-mail.'));
      }
      break;
  }
}

/**
 * Form submission handler for customer search.
 *
 * @, see uc_payafter_invoice_create_form()
 * @ingroup forms
 */
function uc_payafter_invoice_create_form_submit($form, &$form_state) {
    global $user;

  switch ($form_state['values']['customer_type']) {
    case 'search':
      $uid = $form_state['values']['customer']['uid'];
      break;

    case 'create':
      // Create new account.
      $email = trim($form_state['values']['customer']['email']);
      $fields = array(
        'name' => uc_store_email_to_username($email),
        'mail' => $email,
        'pass' => user_password(),
        'status' => variable_get('uc_new_customer_status_active', TRUE) ? 1 : 0,
      );
      $account = user_save(NULL, $fields);
      $uid = $account->uid;

      if ($form_state['values']['customer']['sendmail']) {
        // Manually set the password so it appears in the e-mail.
        $account->password = $fields['pass'];
        drupal_mail('user', 'register_admin_created', $email, language_default(), array('account' => $account), $user->mail);
        drupal_set_message(t('A welcome message has been e-mailed to the new user.'));
      }
      break;
  }
  $created = REQUEST_TIME;
  
  //Get the prevoius invoice id.
  
  $query_invoice = db_select('uc_payafter', 'u')
  ->fields('u', array('invoice_id'))
  ->orderBy('u.invoice_id', 'DESC')
  ->range(0, 1)
  ->execute()
  ->fetchAssoc();
  
  //Intialistion of old_invoice_id variable to 0.
  $old_invoice_id = 0;
  
  //Check either the {uc_payafter} table contains any element or not.
  if (count ($query_invoice) == 0) {
    $old_invoice_id = 1;
  }
  else {
    foreach ($query_invoice as $value) {
      $old_invoice_id = $value;
    }
  }
  
  //Intialistion of old_invoice_id variable to $old_invoice_id + 1.
  $new_invoice_id = $old_invoice_id + 1;
  
  //create the blank invoice
  //db_query("INSERT into {uc_payafter}(user_id,invoice_id,parent_uid,created,status) VALUES(" . $uid . "," . $new_invoice_id . "," . $user->uid . "," . $created . ",0)");
  db_query("INSERT into {uc_payafter}(user_id,invoice_id,parent_uid,created,status) VALUES(:uid, :invoice_id, :parent_uid, :created, 0)",
           array(
            ':uid' => $uid,
            ':invoice_id' => $new_invoice_id,
            ':parent_uid' => $user->uid,
            ':created' => $created,
            ));
  $form_state['redirect'] = 'admin/store/invoice/' . $new_invoice_id . '/edit';
}

/**
 * Displays the order edit screen.
 */
function uc_payafter_invoice_edit_screen($invoice) {
  $output = drupal_get_form('uc_payafter_invoice_edit_form', $invoice);
  return $output;
}

 /**
 * Create the form for add the line Items in to the invoice.
 */
 function uc_payafter_invoice_line_item_edit_form($invoice) {
  $form['line_items'] = array(
    '#type' => 'fieldset',
    '#title' => t('Line items'),
    '#description' => t('Add or Remove Line Items'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $items = _uc_line_item_list();
  foreach ($items as $item) {
    if (isset($item['add_list']) && $item['add_list'] === TRUE) {
      $options[$item['id']] = check_plain($item['title']);
    }
  }
  $form['line_items']['add_new'] = array(
    '#type' => 'select',
    '#title' => 'Add New Item',
    '#options' => $options,
  );
  $form['line_items']['new_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
    '#submit' => array('uc_payafter_pane_line_item_add'),
    '#prefix' => '<span>',
    '#suffix' => '</span>',
  );
  $form['line_items']['item'] = array(
    '#prefix' => '<div id="custom-payafter-old-line_item">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );
  
  $line_items = uc_payafter_old_line_items($invoice);
  if (count($line_items)) {
    foreach  ($line_items as $key => $value) {
      $form['line_items']['item'][$key]['remove'] = array(
        '#type' => 'image_button', 
        '#title' => t('Remove this product.'), 
        '#src' => drupal_get_path('module', 'uc_payafter') . '/images/false-icon.gif', 
        '#button_type' => 'remove', 
        '#submit' => array('uc_payafter_edit_line_item_remove'), 
        '#return_value' => $value['id'],
      );
      $form['line_items']['item'][$key]['title'] = array(
        '#type' => 'textfield',
        '#title' => t('Title'),
        '#title_display' => 'invisible',
        '#default_value' => $value['title'],
      );
      $form['line_items']['item'][$key]['amount'] = array(
        '#type' => 'textfield',
        '#title' => t('Amount'),
        '#title_display' => 'invisible',
        '#default_value' => $value['amount'],
      );
    }
  }
  return $form;
  }

 /**
 * Ajax callback: For add the add product div.
 */
  function uc_payafter_pane_line_item_select($form, $form_state) {
    return $form['line_items']['item'];
  }
/**
 * For add the line item into the particular invoice.
 */
function uc_payafter_pane_line_item_add($form, &$form_state) {
  $sql = "SELECT count(*) FROM {uc_payafter_line_item} WHERE invoice_id=:invoice";
  $sql_result = db_query($sql, array(':invoice' => $form_state['values']['invoice_id']));
  $sql_array = $sql_result->fetchAssoc();
  $count = $sql_array['count(*)'];
  $type = $form_state['values']['add_new'];
  $title = null;
  $invoice = $form_state['values']['invoice_id'];
  if ($type == 'generic') {
    $title = "Empty Line";
  }
  //db_query("INSERT INTO {uc_payafter_line_item}(invoice_id, delta, item_type, title) VALUES(" . $invoice . "," . $count . ",'" . $type . "','" . $title . "')");
  db_query("INSERT INTO {uc_payafter_line_item}(invoice_id, delta, item_type, title) VALUES(:invoice, :count, :type, :title)",
           array(
            ':invoice' => $invoice,
            ':count' => $count,
            ':type' => $type,
            ':title' => $title,
            ));
  
}

/**
 * For get the old line items of an particular invoice.
 */
function uc_payafter_old_line_items($invoice) {
  $query = db_select('uc_payafter_line_item', 'u')
  ->fields('u')
  ->condition('u.invoice_id', $invoice, '=')
  ->execute();
  $line_item = array();
  $i = 0;
  while ($value = $query->fetchAssoc()) {
    $line_item[$i] = $value;
    $i++;
  }
  return $line_item;
}

/**
 * Create the form for add the product in to the invoice.
 *
 * @, see uc_payafter_invoice_edit_form_validate()
 * @, see uc_payafter_invoice_edit_form_submit()
 * @ingroup forms
 */
function uc_payafter_invoice_edit_form($form, &$form_state, $invoice) {
  global $base_url;
  //get the details from the database about this invoice.
  $query2 = db_select('uc_payafter', 'u')
  ->fields('u', array('product_id', 'quantity', 'price_per_product', 'total_price'))
  ->condition('u.invoice_id', $invoice, '=')
  ->execute()
  ->fetchAssoc();
  if ($query2) {
  $query = db_select('uc_payafter', 'u')
  ->fields('u', array('product_id', 'quantity', 'price_per_product', 'total_price'))
  ->condition('u.product_id', '0', '>')
  ->condition('u.invoice_id', $invoice, '=')
  ->condition('u.status', 1, '=')
  ->execute();
    
  //intialise the product array()
  $invoice_products = array();
  $i = 0;
  while ($q_result = $query->fetchAssoc()) {
    $invoice_products[$i] = $q_result;
    $i++;
  }
  $nid = array(0);
  $form['products'] = array(
    '#prefix' => '<div id="custom-payafter-old-product">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );
  $form['invoice_id'] = array(
    '#type' => 'hidden', 
    '#value' => $invoice,
  );
  $i = 0;
  if (count($invoice_products) > 0) {
    $form['products'] += tapir_get_table('uc_payafter_edit_table');
    foreach ($invoice_products as $result) {
      $nid[] = $result['product_id'];
      $node = node_load($result['product_id']);
      $form['products'][$i]['remove'] = array(
        '#type' => 'image_button', 
        '#title' => t('Remove this product.'), 
        '#src' => drupal_get_path('module', 'uc_payafter') . '/images/false-icon.gif', 
        '#button_type' => 'remove', 
        '#submit' => array('uc_payafter_edit_products_remove'), 
        '#return_value' => $result['product_id'],
        );
      $form['products'][$i]['nid'] = array(
        '#type' => 'hidden', 
        '#value' => $result['product_id'],
        );
      $form['products'][$i]['qty'] = array(
        '#type' => 'textfield', 
        '#title' => t('Quantity'), 
        '#title_display' => 'invisible', 
        '#default_value' => $result['quantity'],
        );
      $form['products'][$i]['title'] = array(
        '#type' => 'markup', 
        '#title' => t('Title'), 
        '#title_display' => 'invisible', 
        '#markup' => $node->title, 
        );
      $form['products'][$i]['price'] = array(
        '#type' => 'textfield', 
        '#title' => t('Cost'), 
        '#title_display' => 'invisible', 
        '#default_value' => $result['price_per_product'], 
        '#size' => 10,
        );
      $form['products'][$i]['total_price'] = array(
        '#type' => 'markup', 
        '#title' => t('Price'), 
        '#title_display' => 'invisible', 
        '#markup' => $result['total_price'], 
        );
      $i++;
    }
  }
  else {
    $form['products']['markup'] = array(
      '#type' => 'markup',
      '#markup' => t('There is no products in this invoice'),
      '#prefix' => '<span>',
      '#suffix' => '</span>',
    );
  }
  
  
  //get all the node id form the database except node id which are already add in to the Invoice.  
  $query1 = db_select('node', 'n')
    ->fields('n', array('nid', 'title'))
    ->condition('n.type', 'product', '=')
    ->condition('n.nid', $nid, 'NOT IN')
    ->execute();
  
  $node_array = array();
  while ($result = $query1->fetchAssoc()) {
    $node_array[$result['nid']] = $result['title'];
  }

  $form['add_product_button'] = array(
    '#type' => 'submit',
    '#value' => t('Add product'),
    '#submit' => array('uc_payafter_pane_products_select'),
    '#ajax' => array(
      'callback' => 'uc_payafter_products_ajax_callback',
      'wrapper' => 'custom-payafter-add-product',
    ),
    '#prefix' => '<span>',
    '#suffix' => '</span>',
  );
  
  $form['new_products'] = array(
      '#prefix' => '<div id="custom-payafter-add-product">',
      '#suffix' => '</div>',
      '#tree'   => TRUE,
    );
  $form['line_item'] = uc_payafter_invoice_line_item_edit_form($invoice);
  if ($form_state['products_action'] == 'add_product') {
    
    $form['new_products'] += array(
      '#type' => 'fieldset',
      '#title' => t('Add new Product'),
      '#description' => t('Add New Products in this Invoice'),
    );
    $form['new_products']['list'] = array(
      '#type' => 'select',
      '#options' => $node_array,
      '#size' => 7,
    );
    $form['new_products']['quantity'] = array(
      '#type' => 'textfield',
      '#size' => 7,
      '#title' => t('Enter Quantity'),
      '#maxlength' => 10,
    );
    $form['new_products']['add_product'] = array(
      '#type' => 'submit',
      '#value' => 'Add Product',
      '#validate' => array('uc_payafter_pane_products_add_validate'),
      '#submit' => array('uc_payafter_pane_products_add_submit'),
      );
  }
  
  $form['actions']['submit_changes'] = array(
    '#type' => 'submit',
    '#value' => t('Submit Changes'),
    '#submit' => array('uc_payafter_pane_products_changes_submit'),
  );
  $form['actions']['delete'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => array('uc_payafter_invoice_delete_form_submit'),
  );
  
  }
  else {
    drupal_set_message(t('There is no such a Invoice'));
    drupal_goto($base_url);
  }
  return $form;
}

 /**
 * Ajax callback: For add the add product div.
 */
function uc_payafter_products_ajax_callback($form, &$form_state) {
  return $form['new_products'];
}
/**
 * Form submission handler for Add Product Div.
 *
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
*/
function uc_payafter_pane_products_select($form, &$form_state) {
  $form_state['products_action'] = 'add_product';
  $form_state['rebuild'] = TRUE;
}

/**
 * Form validation handler for Add the product in the Invoice.
 *
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
 */
function uc_payafter_pane_products_add_validate($form, &$form_state) {
  if (empty($form_state['values']['new_products']['list'])) {
    form_set_error('new_product][list', t('Please Select atleast One Product.'));
  }
  if (empty($form_state['values']['new_products']['quantity'])) {
    form_set_error('new_product][quantity', t('Please Enter the Qunatity'));
  }
}
 /**
 * Form submission handler for Add the product in the Invoice.
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
 */
function uc_payafter_pane_products_add_submit($form, &$form_state) {
  global $user;
  $user_id = $user->uid;
  $invoice = $form_state['values']['invoice_id'];
  $nid = $form_state['values']['new_products']['list'];
  $node_info = node_load($nid);
  $uid = db_query("SELECT user_id FROM {uc_payafter} WHERE invoice_id=:invoice_id", array(':invoice_id' => $invoice))->fetchAssoc();
  $qty = $form_state['values']['new_products']['quantity'];
  
  //get the delta value
  $delta = db_query("SELECT COUNT(*) FROM {uc_payafter} WHERE invoice_id=:invoice_id", array(':invoice_id' => $invoice))->fetchAssoc();
  $total_price = $qty * $node_info->sell_price;
  $created = REQUEST_TIME;
  db_query("INSERT INTO {uc_payafter}
           (invoice_id, product_id, delta, status, user_id, quantity, price_per_product, total_price, created, parent_uid)
           VALUES(:invoice_id, :product_id, :delta, 1, :user_id, :quantity, :price_product, :total_price, :created, :parent_uid)",
           array(
            ':invoice_id' => $invoice,
            ':product_id' => $nid,
            ':delta' => $delta['COUNT(*)'],
            ':user_id' => $uid['user_id'],
            ':quantity' => $qty,
            ':price_product' => $node_info->sell_price,
            ':total_price' => $total_price,
            ':created' => $created,
            ':parent_uid' => $user_id,
            ));
  if ($delta['COUNT(*)'] == 1) {
    $user_info = user_load($uid);
    $key = 'key';
    $to = $user_info->mail;
    $from = variable_get('site_mail');
    global $base_url;
    $params = array(
      'subject' => t('A Invoice is created for you.'),
      'body' => t('A Invoice is created for you. You may access to this invoice via this link') . $base_url . '/invoice/' . $invoice . '/view',
    );
    drupal_mail('uc_payafter', $key, $to, language_default(), $params, $from);
  }
}
 
 /**
 * Form submission handler for Make changes in the Invoice like quantity etc.
 *
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
 */
function uc_payafter_pane_products_changes_submit($form, &$form_state) {
  $invoice = $form_state['values']['invoice_id'];
  foreach ($form_state['values']['products'] as $key => $value) {
    $i = $key+1;
    $qty = $form_state['values']['products'][$key]['qty'];
    $price = $form_state['values']['products'][$key]['price'];
    $total_price = $qty * $price;
    //db_query("UPDATE {uc_payafter} SET quantity=" . $qty . ", price_per_product=" . $price . ", total_price=" . $total_price . " WHERE invoice_id=" . $invoice . " and delta=" . $i);
    db_query("UPDATE {uc_payafter} SET quantity=:quantity, price_per_product=:price_product, total_price=:total_price WHERE invoice_id=:invoice_id and delta=:delta",
             array(
              ':quantity' => $qty,
              ':price_product' => $price,
              ':total_price' => $total_price,
              ':invoice_id' => $invoice,
              ':delta' => $i,
              ));
  }
  foreach ($form_state['values']['item'] as $key => $value) {
    $title = $value['title'];
    $amount = $value['amount'];
    //db_query("UPDATE {uc_payafter_line_item} SET title='" . $title . "', amount='" . $amount . "' WHERE invoice_id=" . $invoice . " and delta=" . $key);
    db_query("UPDATE {uc_payafter_line_item} SET title=:title, amount=:amount WHERE invoice_id=:invoice and delta=:key",
             array(
              ':title' => $title,
              ':amount' => $amount,
              ':invoice' => $invoice,
              ':key' => $key,
              ));
  }
}

 /**
 * Form submission handler for remove the product in the Invoice etc.
 *
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
 */
function uc_payafter_edit_products_remove($form, &$form_state) {
  $invoice = $form_state['values']['invoice_id'];
  $query = db_select('uc_payafter', 'u')
    ->fields('u')
    ->condition('u.invoice_id', $invoice, '=')
    ->condition('u.status', 1, '=')
    ->execute();
    
  //get the delta_value or nid of the product which is to be remove form the Invoice.
  $delta = null;
  foreach ($form_state['values']['products'] as $key => $value) {
    if (!empty($value['remove'])) {
      $delta = $key;
    }
  }
  
  //intialise an array
  $invoice_products = array();
  $i = 0;
  while ($result = $query->fetchAssoc()) {
    $invoice_products[$i] = $result;
    $i++;
  }
  
  //delete or unset the nid from the array $invoice_products
  unset($invoice_products[$delta]);
  
  //intialise new array after delete the required product from Invoice.
  $new_products_array = array();
  $i = 0;
  foreach ($invoice_products as $key => $value) {
    $value['delta'] = $i+1;
    $new_products_array[$i] = $value;
    $i++;
  }
  
  //Delete all the Values Regarding this Particular Invoice from Database.
  //db_query("DELETE from {uc_payafter} WHERE invoice_id=" . $invoice . " AND status=1");
  db_query("DELETE from {uc_payafter} WHERE invoice_id=:invoice AND status=1", array(':invoice' => $invoice));
  //Again add the new array in the database.
  foreach ($new_products_array as $key => $value) {
    //$sql = "INSERT INTO {uc_payafter}(invoice_id, user_id, delta, product_id, quantity, price_per_product, total_price, parent_uid, created, status)
    //VALUES(" . $value['invoice_id'] . "," . $value['user_id'] . "," . $value['delta'] . "," . $value['product_id'] . "," . $value['quantity'] . "," . $value['price_per_product'] . "," . $value['total_price'] . "," . $value['parent_uid'] . "," . $value['created'] . "," . $value['status'] . ")";
    $sql = "INSERT INTO {uc_payafter}(invoice_id, user_id, delta, product_id, quantity, price_per_product, total_price, parent_uid, created, status)
    VALUES(:invoice_id, :user_id, :delta, :product_id, :quantity, :price_per_product, :total_price, :parent_uid, :created, :status)";
    db_query($sql, array(
      ':invoice_id' => $value['invoice_id'],
      ':user_id' => $value['user_id'],
      ':delta' => $value['delta'],
      ':product_id' => $value['product_id'],
      ':quantity' => $value['quantity'],
      ':price_per_product' => $value['price_per_product'],
      ':total_price' => $value['total_price'],
      ':parent_uid' => $value['parent_uid'],
      ':created' => $value['created'],
      ':status' => $value['status'],
      ));
  }
}

 /**
 * Form submission handler for remove the Line Item in the Invoice etc.
 *
 * @, see uc_payafter_invoice_edit_form()
 * @ingroup forms
 */
function uc_payafter_edit_line_item_remove($form, &$form_state) {
  $invoice = $form_state['values']['invoice_id'];
  $query = db_select('uc_payafter_line_item', 'u')
    ->fields('u')
    ->condition('u.invoice_id', $invoice, '=')
    ->execute();
    
  //get the delta_value or nid of the product which is to be remove form the Invoice.
  $delta = null;
  foreach ($form_state['values']['item'] as $key => $value) {
    if (!empty($value['remove'])) {
      $delta = $key;
    }
  }
  
  //intialise an array
  $invoice_line_items = array();
  $i = 0;
  while ($result = $query->fetchAssoc()) {
    $invoice_line_items[$i] = $result;
    $i++;
  }
  
  //delete or unset the nid from the array $invoice_products
  unset($invoice_line_items[$delta]);
  
  //intialise new array after delete the required product from Invoice.
  $new_line_item_array = array();
  $i = 0;
  foreach ($invoice_line_items as $key => $value) {
    $value['delta'] = $i+1;
    $new_line_item_array[$i] = $value;
    $i++;
  }
  
  //Delete all the Values Regarding this Particular Invoice from Database.
  db_query("DELETE FROM {uc_payafter_line_item} WHERE invoice_id = :invoice", array(':invoice' => $invoice));
  
  //Again add the new array in the database.
  foreach ($new_line_item_array as $key => $value) {
    //$sql = "INSERT INTO {uc_payafter_line_item}(invoice_id, delta, item_type, title, amount)
    //VALUES(" . $value['invoice_id'] . "," . $value['delta'] . ",'" . $value['item_type'] . "','" . $value['title'] . "'," . $value['amount'] . ")";
    $sql = "INSERT INTO {uc_payafter_line_item}(invoice_id, delta, item_type, title, amount) VALUES(:invoice, :delta, :item_type, :title, :amount)";
    db_query($sql, array(
      ':invoice' => $value['invoice_id'],
      ':delta' => $value['delta'],
      ':item_type' => $value['item_type'],
      ':title' => $value['title'],
      ':amount' => $value['amount'],
    ));
  }
}

 /**
 * Displays the Invoice view screen.
 */
function uc_payafter_invoice_view() {
  $output = array_merge(uc_payafter_invoice_view_filter(), uc_payafter_invoice_view_filter_result());
  return $output;
}
 /**
 * For add the filter in the Invoice View Screen
 */
function uc_payafter_invoice_view_filter() {
  $output1 = drupal_get_form('uc_payafter_invoice_view_filter_form');
  return $output1;
}
 /**
 * For add the filter in the Invoice View Screen
 * 
 * @, see uc_payafter_invoice_view_filter_form_validate()
 * @, see uc_payafter_invoice_view_filter_form_submit()
 * @ingroup forms
 */
function uc_payafter_invoice_view_filter_form($form, &$form_state) {
  $form['filter'] = array(
    '#prefix' => "<div id = 'view_filter_form'>",
    '#suffix' => "</div>",
    '#tree' => TRUE,
  );
  $form['filter'] += array(
    '#type' => 'fieldset',
    '#title' => t('View Filter'),
    '#description' => t('Filter Acc. to Invoice Id and Status'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['filter']['invoice_id'] = array(
    '#type' => 'textfield',
    '#title' => 'invoice Id',
    '#size' => 7,
    '#maxlength' => 255,
  );
  $options = array(
    '0' => '--Select--',
    '1' => 'Pending',
    '2' => 'Complete',
  );
  $form['filter']['status'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => 0,
  );
  
  $form['filter']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
    //'#validate' => 'uc_payafter_invoice_view_filter_form_validate',
    //'#submit' => 'uc_payafter_invoice_view_filter_form_submit',
  );
  return $form;
}

 /**
 * Form validation handler for view the Invoice.
 *
 * @, see uc_payafter_invoice_view_filter_form()
 * @ingroup forms
 */
function uc_payafter_invoice_view_filter_form_validate($form, &$form_state) {
  if (empty($form_state['values']['filter']['invoice_id']) && empty($form_state['values']['filter']['status'])) {
    form_set_error('filter][invoice_id', t('Please select atleast one filter value'));
  }
}

 /**
 * Form Submit handler for view the Invoice.
 *
 * @, see uc_payafter_invoice_view_filter_form()
 * @ingroup forms
 */
function uc_payafter_invoice_view_filter_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] && $form_state['values']['op'] == 'Filter') {
    $_SESSION['view_form_values'] = $form_state['values'];
  }
  else {
    unset($_SESSION['view_form_values']);
  }
}
 /**
 * Make where clause for the uc_payafter_invoice_view_filter_result.
 * @, see uc_payafter_invoice_view_filter_result
 */
function uc_payafter_invoice_view_filter_where_clause() {
  $values = array();
  $result = array(
    'invoice_id' => 0,
    'status' => 0,
  );
  if (isset($_SESSION["view_form_values"])) {
    $values = $_SESSION["view_form_values"];
    if (!empty($values['filter']['invoice_id'])) {
      $result['invoice_id'] = $values['filter']['invoice_id'];
    }
    if (!empty($values['filter']['status'])) {
      $result['status'] = $values['filter']['status'];
    }
  }
  unset($_SESSION["view_form_values"]);
  return $result;
}

 /**
 * Genrate the result according to the filter form.
 * @, see uc_payafter_invoice_view_filter_form()
 * @, see uc_payafter_invoice_view_filter_where_clause()
 */
function uc_payafter_invoice_view_filter_result() {
  global $base_url;
  $where = uc_payafter_invoice_view_filter_where_clause();
  $result = array();
  $where_clause = " where 1=1";
  if ($where['invoice_id']) {
    $where_clause .= " and invoice_id=" . $where['invoice_id'];
  }
  if ($where['status']) {
    $where_clause .= " and status=" . $where['status'];
  }
  $status = null;
  $sql = db_query("SELECT * FROM {uc_payafter}" . $where_clause . " GROUP BY invoice_id");
  $result['products'] = array(
    '#prefix' => '<div id="custom-payafter-view-product">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );
  $result['products'] += tapir_get_table('uc_payafter_view_table');
  $i = 0;
  while ($sql_result = $sql->fetchAssoc()) {
  if (($sql_result['c_status'] == 0)) {
    $status = 'Pending';
  }
  elseif ($sql_result['c_status'] == 1) {
    $status = 'Complete';
  }
    // Get the Total Price of Invoice
    $sql1 = db_query('SELECT SUM(total_price) FROM {uc_payafter} WHERE invoice_id=:invoice', array(':invoice' => $sql_result['invoice_id']))->fetchAssoc();
    $uid = $sql_result['user_id'];
    $user_info = user_load($uid);
    $result['products'][$i]['actions'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/view"><img alt="View Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_view.png" typeof="foaf:Image"></a><a title="Edit Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/edit"><img alt="Edit Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_edit.png" typeof="foaf:Image"></a><a title="Delete Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/delete"><img alt="Delete Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_delete.png" typeof="foaf:Image"></a>',
    );
    $result['products'][$i]['invoice_id'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/view">' . $sql_result['invoice_id'] . '</a>',
    );
    $result['products'][$i]['customer'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View User Profile" href="' . $base_url . '/user/' . $sql_result['user_id'] . '">' . $user_info->name . '</a>',
    );
    $result['products'][$i]['total_price'] = array(
      '#type' => 'markup',
      '#markup' => $sql1['sum(total_price)'],
    );
    $result['products'][$i]['date'] = array(
      '#type' => 'markup',
      '#markup' => date('d/m/Y', $sql_result['created']),
    );
    $result['products'][$i]['status'] = array(
      '#type' => 'markup',
      '#markup' => $status,
    );
    $i++;
  }
  return $result;
}

 /**
 * Genrate the Screen for View the Particular Invoice.
 */
function uc_payafter_particular_invoice_view() {
  global $base_url;
  $invoice = arg(3);
  $query = db_select('uc_payafter', 'u')
    ->fields('u')
    ->condition('u.invoice_id', $invoice, '=')
    ->execute()
    ->fetchAll();
  if (count($query)>0) {
    $query1 = db_select('uc_payafter', 'u')
      ->fields('u')
      ->condition('u.status', '1', '=')
      ->condition('u.invoice_id', $invoice, '=')
      ->execute()
      ->fetchAll();
    $result['products'] = array(
      '#prefix' => '<div id="custom-payafter-view-invoice-product">',
      '#suffix' => '</div>',
      '#tree'   => TRUE,
    );
    $result['products'] += tapir_get_table('uc_payafter_particular_view_table');
    $total_invoice = 0;
    foreach ($query1 as $key => $value) {
      $total_invoice += $value->total_price;
      $node_info = node_load($value->product_id);
      $result['products'][$key]['qty'] = array(
        '#type' => 'markup',
        '#markup' => $value->quantity,
      );
      $result['products'][$key]['product'] = array(
        '#type' => 'markup',
        '#markup' => '<a href = "' . $base_url . '/node/' . $node_info->nid . '">' . $node_info->title . '</a>',
      );
      $result['products'][$key]['sku'] = array(
        '#type' => 'markup',
        '#markup' => $node_info->model,
      );
      $result['products'][$key]['price'] = array(
        '#type' => 'markup',
        '#markup' => $value->price_per_product,
      );
      $result['products'][$key]['total'] = array(
        '#type' => 'markup',
        '#markup' => $value->total_price,
      );
    }
    $result['total'] = array(
      '#prefix' => '<div id="custom-payafter-view-invoice-total">',
      '#suffix' => '</div>',
      '#tree'   => TRUE,
    );
    $result['total'] += array(
      '#type' => 'fieldset',
    );
    $result['total']['invoice'] = array(
      '#type' => 'markup',
      '#markup' => '<b>Subtotal</b>: ' . $total_invoice . '<br/>',
    );
    $query2 = db_select('uc_payafter_line_item', 'u')
      ->fields('u')
      ->condition('u.invoice_id', $invoice, '=')
      ->execute()
      ->fetchAll();
    if (count($query2)>0) {
      foreach ($query2 as $key => $value) {
        $total_invoice += $value->amount;
        $result['total']['line-item' . $key] = array(
           '#type' => 'markup',
           '#markup' => '<b>' . $value->title . '</b> : ' . $value->amount . '<br/>',
        );
      }
    }
    $result['total']['invoice-total'] = array(
       '#type' => 'markup',
       '#markup' => '<b>Total</b> : ' . $total_invoice,
    );
  }
  else {
    drupal_set_message(t('There is no such Invoice exist.'));
    drupal_goto($base_url);
  }
  return $result;
}

 /**
 * Confirmation form to delete an order.
 *
 * @, see uc_payafter_invoice_delete_form_submit()
 * @ingroup forms
 */
function uc_payafter_invoice_delete_form($form, &$form_state) {
  $invoice = $form_state['build_info']['args'][0];
  $form['invoice_id'] = array(
    '#type' => 'value',
    '#value' => $invoice,
  );
  return confirm_form($form, t('Are you sure you want to delete Invoice @invoice_id?', array('@invoice_id' => $invoice)), 'admin/store/invoice/view', NULL, t('Delete'));
}

 /**
 * Form submission handler for uc_payafter_invoice_delete_form.
 *
 * @, see uc_payafter_invoice_delete_form()
 */
function uc_payafter_invoice_delete_form_submit($form, &$form_state) {
  // Delete the specified order.
  $invoice = $form_state['values']['invoice_id'];
  db_query('DELETE FROM {uc_payafter} WHERE invoice_id=:invoice', array(':invoice' => $invoice));
  db_query('DELETE FROM {uc_payafter_line_item} WHERE invoice_id=:invoice', array(':invoice' => $invoice));
  // Display a message to the user and return to the Invoice admin page.
  drupal_set_message(t('Invoice @invoice_id completely removed from the database.', array('@invoice_id' => $form_state['values']['invoice_id'])));

  $form_state['redirect'] = 'admin/store/invoice';
}

 /**
 * Genrate the Screen for Search the Particular Invoice.
 */
function uc_payafter_invoice_search() {
  $output = array_merge(drupal_get_form('uc_payafter_invoice_search_form'), uc_payafter_invoice_search_result());
  return $output;
}

 /**
 * Form submit handler.
 * @, see uc_payafter_invoice_search_form()
 * @, see uc_payafter_invoice_search_where()
 */
function uc_payafter_invoice_search_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] && $form_state['values']['op'] == 'Submit') {
    $_SESSION['search_values'] = $form_state['values'];
  }
  elseif ($form_state['values']['op'] && $form_state['values']['op'] == 'Clear') {
    unset($_SESSION['search_values']);
  }
}

/**
 * Form creation for Search Invoice.
 *
 * @, see uc_payafter_invoice_search_form_submit()
 * @, see uc_payafter_invoice_search_where()
 */
function uc_payafter_invoice_search_form($form, &$form_state) {
  $form = array();
  $form['search'] = array(
    '#prefix' => "<div id = 'invoice-search-form'>",
    '#suffix' => '</div>',
    '#tree' => TRUE,
  );
  $form['search'] += array(
    '#type' => 'fieldset',
    '#title' => 'Search Invoice',
    '#description' => t('Search an Invoice'),
    '#collapsible' => TRUE,
  );
  $options = array(
    '2' => '--select--',
    '0' => 'Pending',
    '1' => 'Complete',
  );
  if (!empty($_SESSION['search_values'])) {
    $value = $_SESSION['search_values'];
    $form['search']['invoice_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter Invoice Id',
      '#description' => t('Search by an Invoice Id'),
      '#size' => 7,
      '#default_value' => $value['search']['invoice_id'],
    );
    $form['search']['user_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter User Id',
      '#description' => t('Enter User Id for which Invoice was created.'),
      '#size' => 7,
      '#default_value' => $value['search']['user_id'],
    );
    $form['search']['auther_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter Author Id',
      '#description' => t('Enter User Id who created the Invoice.'),
      '#size' => 7,
      '#default_value' => $value['search']['auther_id'],
    );
    $form['search']['created_start'] = array(
      '#type' => 'date',
      '#title' => 'Enter Start Date',
      '#description' => t('Enter Start Date.'),
      '#size' => 7,
      '#default_value' => $value['search']['created_start'],
    );
    $form['search']['created_end'] = array(
      '#type' => 'date',
      '#title' => 'Enter End Date',
      '#description' => t('Enter End Date.'),
      '#size' => 7,
      '#default_value' => $value['search']['created_end'],
    );
    $form['search']['status'] = array(
      '#type' => 'select',
      '#title' => 'Enter Status',
      '#description' => t('Enter Status of the Invoice.'),
      '#options' => $options,
      '#default_value' => $value['search']['status'],
    );    
  }
  else {
    $form['search']['invoice_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter Invoice Id',
      '#description' => t('Search by an Invoice Id'),
      '#size' => 7,
    );
    $form['search']['user_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter User Id',
      '#description' => t('Enter User Id for which Invoice was created.'),
      '#size' => 7,
    );
    $form['search']['auther_id'] = array(
      '#type' => 'textfield',
      '#title' => 'Enter Author Id',
      '#description' => t('Enter User Id who created the Invoice.'),
      '#size' => 7,
    );
    $form['search']['created_start'] = array(
      '#type' => 'date',
      '#title' => 'Enter Start Date',
      '#date_format' => 'd/m/Y',
      '#description' => t('Enter Start Date.'),
      '#size' => 7,
    );
    $form['search']['created_end'] = array(
      '#type' => 'date',
      '#title' => 'Enter End Date',
      '#date_format' => 'd/m/Y',
      '#description' => t('Enter End Date.'),
      '#size' => 7,
    );
    
    $form['search']['status'] = array(
      '#type' => 'select',
      '#title' => 'Enter Status',
      '#description' => t('Enter Status of the Invoice.'),
      '#options' => $options,
      '#default_value' => 2,
    );
  }
  $form['search']['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Submit',
  );
  $form['search']['clear'] = array(
    '#type' => 'submit',
    '#value' => 'Clear',
  );
  return $form;
}

 /**
 * Create the where clause for the search form.
 * @, see uc_payafter_invoice_search_form()
 */
function uc_payafter_invoice_search_where() {
  $where_clause = " WHERE 1=1";
  if ($_SESSION['search_values']) {
    $value = $_SESSION['search_values'];
    if (!empty($value['search']['invoice_id'])) {
      $where_clause .= " AND invoice_id=" . $value['search']['invoice_id'];
    }
    if (!empty($value['search']['user_id'])) {
      $where_clause .= " AND user_id=" . $value['search']['user_id'];
    }
    if (!empty($value['search']['auther_id'])) {
      $where_clause .= " AND parent_uid=" . $value['search']['auther_id'];
    }
    if (!empty($value['search']['status']) && $value['search']['status'] == 1) {
      $where_clause .= " AND c_status=1";
    }
    elseif (!empty($value['search']['status']) && $value['search']['status'] == 0) {
      $where_clause .= " AND c_status=0";
    }
    if (!empty($value['search']['created_start'])) {
      $date = $value['search']['created_start']['year'] . "/" . $value['search']['created_start']['month'] . "/" . $value['search']['created_start']['day'];
      $date_time = strtotime($date);
      $now = strtotime(date('Y-m-d'));
      if ($now != $date_time) {
        $where_clause .= " AND created>=" . $date_time;
      }
    }
    if (!empty($value['search']['created_end'])) {
      $date = $value['search']['created_end']['year'] . "/" . $value['search']['created_end']['month'] . "/" . $value['search']['created_end']['day'];
      $date_time = strtotime($date);
      $now = strtotime(date('Y-m-d'));
      if ($now != $date_time) {
        $where_clause .= " AND created<=" . $date_time;
      }
    }
  }
  return $where_clause;
}

 /**
 * Create the result for the Search Form
 */
function uc_payafter_invoice_search_result() {
  $where = uc_payafter_invoice_search_where();
  global $base_url;
  $sql = db_query("SELECT * from {uc_payafter}" . $where . " GROUP BY invoice_id");
  $result = array();
  $result['products'] = array(
    '#prefix' => '<div id="custom-payafter-search-product">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );
  $result['products'] += tapir_get_table('uc_payafter_view_table');
  $i = 0;
  while ($sql_result = $sql->fetchAssoc()) {
    if (($sql_result['c_status'] == 0)) {
      $status = 'Pending';
    }
    elseif ($sql_result['c_status'] == 1) {
      $status = 'Complete';
    }
    // Get the Total Price of Invoice
    $sql1 = db_query('SELECT SUM(total_price) FROM {uc_payafter} WHERE invoice_id=:invoice', array(':invoice' => $sql_result['invoice_id']))->fetchAssoc();
    $uid = $sql_result['user_id'];
    $user_info = user_load($uid);
    $result['products'][$i]['actions'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/view"><img alt="View Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_view.png" typeof="foaf:Image"></a><a title="Edit Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/edit"><img alt="Edit Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_edit.png" typeof="foaf:Image"></a><a title="Delete Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/delete"><img alt="Delete Invoice" src="' . $base_url . '/sites/all/modules/uc_payafter/images/order_delete.png" typeof="foaf:Image"></a>',
    );
    $result['products'][$i]['invoice_id'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View Invoice" href="' . $base_url . '/admin/store/invoice/' . $sql_result['invoice_id'] . '/view">' . $sql_result['invoice_id'] . '</a>',
    );
    $result['products'][$i]['customer'] = array(
      '#type' => 'markup',
      '#markup' => '<a title="View User Profile" href="' . $base_url . '/user/' . $sql_result['user_id'] . '">' . $user_info->name . '</a>',
    );
    $result['products'][$i]['total_price'] = array(
      '#type' => 'markup',
      '#markup' => $sql1['sum(total_price)'],
    );
    $result['products'][$i]['date'] = array(
      '#type' => 'markup',
      '#markup' => date('d/m/Y', $sql_result['created']),
    );
    $result['products'][$i]['status'] = array(
      '#type' => 'markup',
      '#markup' => $status,
    );
    $i++;
  }
  return $result;
}